package com.lee.jbuilder.syntaxchecker;

import java.awt.Color;
import java.util.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.tree.DefaultMutableTreeNode;

import com.borland.primetime.editor.*;
import com.borland.primetime.ide.BrowserIcons;
import com.borland.jbuilder.node.*;
import com.borland.jbuilder.node.java.*;

import com.lee.util.swing.text.SquigglePainter;
import com.lee.jbuilder.util.editor.*;


abstract public class SourceErrorWatcher extends HighlightContextListener {
  public static final int HIGHLIGHT_WIDTH = 3;

  /** The Icon used to draw in the Gutter Margin */
  public static final LineMark LINE_MARK_ERROR = new GutterMark( true, JavaStructureIcons.ICON_ERROR );

  // CONSTRUCTORS

  public SourceErrorWatcher( TextContext context ) {
    super( context );
  }

  // ABSTRACT METHODS

  abstract protected SourceError checkForError( TreeNode node, Object userObject );

  abstract boolean isErrorFolder( TreeNode node, Object userObject );

  // EDITORWATCHER METHODS

  public void sourceChanged() {
    removeHighlighting();
    getEditor().repaint();
  }

  public long getUpdateDelay() {
    return SyntaxChecker.getErrorDelay();
  }

  // HIGHLIGHTCONTEXTLISTENER METHODS

  protected Collection getHighlightInstructions() {
    List instructions = new ArrayList( 30 );
    TreeNode root = (TreeNode) getModel().getRoot();

    for( int i = 0; i < root.getChildCount(); i++ ) {
      TreeNode folder = root.getChildAt( i );

      if( isErrorFolder( folder, getUserObject( folder ) ) ) {
        expandErrorFolder( folder );

        for( int j = 0; j < folder.getChildCount(); j++ ) {
          TreeNode node = folder.getChildAt( j );
          HighlightInstruction instr = checkNode( node, getUserObject( node ) );
          if( instr != null )
            instructions.add( instr );
        }
      }
    }

    Collections.sort( instructions, HighlightInstruction.LINE_COLUMN_SORTER );

    if( getMaxInstructions() < instructions.size() )
      return instructions.subList( 0, getMaxInstructions() );

    return instructions;
  }

  // HELPER METHODS

  static protected Object getUserObject( Object node ) {
    return node instanceof DefaultMutableTreeNode ? ( (DefaultMutableTreeNode) node ).getUserObject() : null;
  }

  protected void expandErrorFolder( final TreeNode folder ) {
    if( folder instanceof DefaultMutableTreeNode && SyntaxChecker.autoExpandErrorFolder() ) {
      javax.swing.SwingUtilities.invokeLater( new Runnable() {
        public void run() {
          try {
            getTree().expandPath( new TreePath( ( (DefaultMutableTreeNode) folder).getPath() ) );
          } catch( Exception e ) {}
        }
      } );
    }
  }

  protected HighlightInstruction checkNode( TreeNode node, Object userObject ) {
    SourceError err = checkForError( node, userObject );
    return err == null ? null : createHighlightInstruction( err );
  }

  protected HighlightInstruction createHighlightInstruction( SourceError err ) {
    GutterMark mark = SyntaxChecker.useLineMarks() ?
                          new GutterMark( JavaStructureIcons.ICON_ERROR ) : null;

    Highlighter.HighlightPainter painter = SyntaxChecker.useUnderline() ?
                          new SquigglePainter( SyntaxChecker.getUnderlineColor(), HIGHLIGHT_WIDTH ) :
                          null;

    String toolTip = SyntaxChecker.useToolTips() ? err.msg : null;

    return new HighlightInstruction( getEditor(), err.line, err.column, HIGHLIGHT_WIDTH, toolTip,
                  mark, painter );
  }

  protected int getMaxInstructions() {
    return SyntaxChecker.getMaxErrorCount();
  }

}