package com.lee.jbuilder.util.editor;

import java.util.*;
import javax.swing.text.*;

import com.borland.primetime.editor.LineMark;
import com.borland.primetime.editor.EditorPane;

public class HighlightInstruction {

  public static final Comparator LINE_COLUMN_SORTER =
    new Comparator() {
      public int compare( Object obj1, Object obj2 ) {
        HighlightInstruction instr1 = (HighlightInstruction) obj1;
        HighlightInstruction instr2 = (HighlightInstruction) obj2;
        int lineDiff = instr1.getLine() - instr2.getLine();
        return lineDiff == 0 ? instr1.getColumn() - instr2.getColumn() : lineDiff;
      }
  };

  // MEMBER VARIABLES

  protected String toolTip;
  protected Highlighter.HighlightPainter painter;
  protected LineMark mark;
  protected int p0, p1;
  protected EditorPane editor;

  public HighlightInstruction( EditorPane editor, int line, int column, int width, String toolTip,
                               LineMark mark, Highlighter.HighlightPainter painter )
  {
    this.editor = editor;
    this.toolTip = toolTip;
    this.mark = mark;
    this.painter = painter;
    calcPosition( line, column, width );
  }

  public HighlightInstruction( EditorPane editor, int start, int end, String toolTip,
                               LineMark mark, Highlighter.HighlightPainter painter )
  {
    this.editor = editor;
    this.toolTip = toolTip;
    this.mark = mark;
    this.painter = painter;
    this.p0 = start;
    this.p1 = end;
  }

  // API

  public int getLine() {
    return editor.getLineNumber( getStartPosition() );
  }

  public int getColumn() {
    return editor.getColumnNumber( getEndPosition() );
  }

  public int getWidth() {
    return getEndPosition() - getStartPosition();
  }

  public LineMark getLineMark() {
    return mark;
  }

  public String getToolTip() {
    return toolTip;
  }

  public Highlighter.HighlightPainter getHighlightPainter() {
    return painter;
  }

  public int getStartPosition() {
    return p0;
  }

  public int getEndPosition() {
    return p1;
  }

  // HELPER METHODS
  protected void calcPosition( int line, int column, int width ) {
    String text = editor.getTextLine( line );

    if ( text != null && column < text.length() && column > width ) {
      p1 = editor.calcCaretPosition( line, column );
      p0 = p1 - width;
    } else {
      p0 = editor.calcCaretPosition( line, 1 );
      p1 = p0 + width;

      // Ensure that p1 is not on the next line
      while ( editor.getLineNumber( p1 ) > editor.getLineNumber( p0 ) )
        p1--;
    }

  }

}