package com.lee.jbuilder.util.editor;

import java.util.*;

import com.borland.primetime.editor.EditorPane;
import com.borland.primetime.ide.*;
import com.borland.primetime.node.*;
import com.borland.primetime.node.TextFileNode;

public class TextContextManager {

  // STATIC MEMBER VARIABLES

  protected static final Comparator UPDATE_DELAY_COMPARATOR =
    new Comparator() {
      public int compare( Object obj1, Object obj2 ) {
        return (int) ( ( ( (TextContextListener) obj2 ).getUpdateDelay() ) - ( (TextContextListener) obj1 ).getUpdateDelay() ) ;
      }
    };

  protected static boolean initialized = false;

  /** A Collection of registered TextContextListenerFactory(s) */
  protected static Collection factories = new ArrayList( 5 );

  protected static List contexts = Collections.synchronizedList( new LinkedList() );

  // API STATIC METHODS

  public static void registerFactory( TextContextListenerFactory factory ) {
    initialize();
    factories.add( factory );
  }

  public static TextContext getTextContext( Context context ) {
    synchronized( contexts ) {
      int index = contexts.indexOf( context );
      if( index >= 0 )
        return (TextContext) contexts.get( index );
    }
    return null;
  }

  public static TextContext getTextContext( EditorPane editor ) {
    synchronized( contexts ) {
      for( Iterator i = contexts.iterator(); i.hasNext(); ) {
        TextContext context = (TextContext) i.next();
        if( context.getEditor().equals( editor ) )
          return context;
      }
    }
    return null;
  }

  // HELPER METHODS

  protected static void initialize() {
    if( initialized )
      return;

    Browser.addStaticBrowserListener(new BrowserAdapter() {

      public void browserViewerActivated(Browser browser, Node node, NodeViewer nodeViewer) {
        browserNodeActivated( browser, node );
      }

      public void browserNodeActivated(Browser browser, Node node ) {
//        System.err.println( "browserNodeActivated()" + node );
        Collection factories = getFactories( node );
        if( ! factories.isEmpty() ) {
          TextContext context = new TextContext( browser, node );
          watchContext( context );
        }
      }

      public void browserNodeClosed(Browser browser, Node node ) {
//        System.err.println( "browserNodeClosed()" + node );
        unwatchContext( new TextContext( browser, node ) );
      }
    } );

    initialized = true;
  }

  protected static Collection getFactories( Node node ) {
    if( node == null || !( node instanceof TextFileNode ) )
      return Collections.EMPTY_LIST;

    List list = new ArrayList( factories.size() );

    for( Iterator i = factories.iterator(); i.hasNext() ; ) {
      TextContextListenerFactory factory =  (TextContextListenerFactory) i.next();
      if( factory.accepts( (TextFileNode) node ) )
        list.add( factory );
    }

    return list;
  }

  protected static void watchContext( TextContext context ) {
    synchronized( contexts ) {
      int index = contexts.indexOf( context );
      if( index >= 0 )
        ( (TextContext) contexts.get( index ) ).init();
      else if( context.isReady() ) {
//      System.err.println( "watchContext( " + context.getNode() + " ) " );

        for( Iterator i = getFactories( context.getNode() ).iterator(); i.hasNext(); )
          context.addListener( ( (TextContextListenerFactory) i.next() ).create( context ) );

        contexts.add( context );
        context.init();
      }
    }
  }

  protected static void unwatchContext( TextContext context ) {
    synchronized( contexts ) {
      int index = contexts.indexOf( context );
      if( index < 0 )
        return;

      // Context from List
      context = (TextContext) contexts.get( index );
      context.close();
      contexts.remove( index );
    }
  }

}